/*
 * main.h
 *
 * PWLib application header file for opengk
 *
 * Copyright 2001 Equivalence Pty. Ltd.
 *
 * $Log: main.h,v $
 * Revision 1.4  2002/05/06 00:48:46  robertj
 * Sizeable rewrite of gatekeeper server code to make more bulletproof against
 *   multithreaded operation. Especially when using slow response/RIP feature.
 * Also changed the call indexing to use call id and direction as key.
 *
 * Revision 1.3  2002/02/04 05:19:40  robertj
 * Added call status display on HTTP.
 * Added test for slow ARQ
 *
 * Revision 1.2  2001/08/10 11:03:49  robertj
 * Major changes to H.235 support in RAS to support server.
 *
 * Revision 1.1  2001/08/03 04:48:57  robertj
 * First cut of a gatekeeper
 *
 */

#ifndef _OpenGk_MAIN_H
#define _OpenGk_MAIN_H


#include <ptclib/httpsvc.h>
#include <h323.h>
#include <gkserver.h>


class MyEndPoint;
class MyGatekeeperServer;


class MyGatekeeperCall : public H323GatekeeperCall
{
    PCLASSINFO(MyGatekeeperCall, H323GatekeeperCall);
  public:
    MyGatekeeperCall(
      MyGatekeeperServer & server,
      const OpalGloballyUniqueID & callIdentifier, /// Unique call identifier
      Direction direction
    );

    virtual H323GatekeeperRequest::Response OnAdmission(
      H323GatekeeperARQ & request
    );
};



class MyGatekeeperServer : public H323GatekeeperServer
{
    PCLASSINFO(MyGatekeeperServer, H323GatekeeperServer);
  public:
    MyGatekeeperServer(MyEndPoint & ep);

    // Overrides
    virtual H323GatekeeperCall * CreateCall(
      const OpalGloballyUniqueID & callIdentifier,
      H323GatekeeperCall::Direction direction
    );

    // new functions
    BOOL Initialise(PConfig & cfg, PConfigPage * rsrc);

  private:
    MyEndPoint & endpoint;
};


class MyEndPoint : public H323EndPoint
{
    PCLASSINFO(MyEndPoint, H323EndPoint);
  public:
    MyEndPoint();
    ~MyEndPoint();

    // overrides from H323EndPoint
    H323Connection * CreateConnection(unsigned callReference);

    // new functions
    BOOL Initialise(PConfig & cfg, PConfigPage * rsrc);
    PString OnLoadEndPointStatus(const PString & htmlBlock);
    PString OnLoadCallStatus(const PString & htmlBlock);

  protected:
    MyGatekeeperServer gatekeeperServer;
};


class OpenGk : public PHTTPServiceProcess
{
  PCLASSINFO(OpenGk, PHTTPServiceProcess)

  public:
    OpenGk();
    void Main();
    BOOL OnStart();
    void OnStop();
    void OnControl();
    void OnConfigChanged();
    PString GetPageGraphic();
    BOOL Initialise(const char * initMsg);

    static OpenGk & Current() { return (OpenGk &)PProcess::Current(); }

    MyEndPoint & GetEndPoint() const { return *endpoint; }

  protected:
    MyEndPoint * endpoint;
};


class MainStatusPage : public PServiceHTTPString
{
  PCLASSINFO(MainStatusPage, PServiceHTTPString);

  public:
    MainStatusPage(OpenGk & app, PHTTPAuthority & auth);
    
    virtual BOOL Post(
      PHTTPRequest & request,
      const PStringToString &,
      PHTML & msg
    );
  
  private:
    OpenGk & app;
};


#endif  // _OpenGk_MAIN_H


// End of File ///////////////////////////////////////////////////////////////
